/* $Id: preparse.c,v 1.6 2002/01/25 14:09:27 emile Exp $
Copyright (C) 1999 Jukka Zitting <jukka.zitting@iki.fi>
Copyright (C) 2000 The Midgard Project ry
Copyright (C) 2000 Emile Heyns, Aurora SA <emile@iris-advies.com>
Copyright (C) 2001 Emile Heyns

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU Lesser General Public License as published
by the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "mgd_internal.h"
#include "mgd_oop.h"
#include <glib.h>
#include <midgard/apache.h>
#include <midgard/pageresolve.h>

static const char* mgd_preparse_get_element(const char *name, void *userdata)
{
table *elements = (table*)userdata;

   return ap_table_get(elements, name);
}

static void mgd_preparse_buffer_add(char *text, int len, void *userdata)
{
GByteArray *buffer = (GByteArray*)userdata;

   g_byte_array_append(buffer, (guint8*)text, len);
}

GByteArray *mgd_preparse_string(char *phpcode)
{
mgd_parser_itf_t itf;
GByteArray *buffer = g_byte_array_new();

   itf.output.func = mgd_preparse_buffer_add;
   itf.output.userdata = buffer;
   itf.get_element.func = mgd_preparse_get_element;
   itf.get_element.userdata = mgd_rcfg()->elements;
   mgd_preparse_buffer(phpcode, &itf);

   /*[eeh] add terminating '\0' */
   g_byte_array_append(buffer, "", 1);

   return buffer;
}

MGD_FUNCTION(string, preparse, (string phpcode))
{
zval **phpcode;
GByteArray *buffer;

   CHECK_MGD;

   if (ZEND_NUM_ARGS() != 1) { WRONG_PARAM_COUNT; }
   if (zend_get_parameters_ex(1, &phpcode) != SUCCESS) { WRONG_PARAM_COUNT; }

   convert_to_string_ex(phpcode);

   buffer = mgd_preparse_string((*phpcode)->value.str.val);

   RETVAL_STRING(buffer->data, 1);
   g_byte_array_free(buffer, TRUE);
}

MGD_FUNCTION(string, format, (string value, string formatspec))
{
midgard_pool *pool;
zval **value, **formatter;
char *fmt = "";
char fmtspec[3];

   CHECK_MGD;

   switch (ZEND_NUM_ARGS()) {
      case 2:
         if (zend_get_parameters_ex(2, &value, &formatter) != SUCCESS) {
            WRONG_PARAM_COUNT;
         }
         break;

      case 1:
         if (zend_get_parameters_ex(1, &value) != SUCCESS) {
            WRONG_PARAM_COUNT;
         }
         fmt = NULL;
         break;

      default:
         WRONG_PARAM_COUNT;
         break;
   }

   convert_to_string_ex(value);
   if (fmt) {
      convert_to_string_ex(formatter);
      fmt = (*formatter)->value.str.val;
   } else {
      fmt = "";
   }

   pool = mgd_alloc_pool();

   strcpy(fmtspec, "$?");
   fmtspec[1] = fmt[0];

   switch (fmtspec[1]) {
      case '\0': /* default */
      case 'T':
         fmtspec[1] = 'p';
         break;
      case 't':
         fmtspec[1] = 'h';
         break;
      case 'h':
         fmtspec[1] = 'H';
         break;

      case 'u':
      case 'f':
      case 'F':
         break;

      default:
         fprintf(stderr, "Unknown formatter '%c'\n", fmtspec[1]);
         fmtspec[1] = 'p';
         break;
   }

   RETVAL_STRING(mgd_format(mgd_handle(), pool, fmtspec,
      (*value)->value.str.val), 1);

   mgd_free_pool(pool);
}
